@@ -18,7 +18,7 @@
     Boston, MA 02110-1301, USA.
 */
 
-#include <QtTest/QtTest>
+#include <QtTest>
 #include <accessmanager.h>
 #include <QNetworkReply>
 
@@ -50,9 +50,6 @@ private Q_SLOTS:
 
     void testPut()
     {
-#if defined(USE_QNAM) && QT_VERSION <= QT_VERSION_CHECK(5, 5, 0)
-        QSKIP("This test is broken with Qt < 5.5, the fix is 9286a8e5dd97c5d4d7e0ed07a73d4ce7240fdc1d in qtbase");
-#endif
         const QString aDir = QStandardPaths::writableLocation(QStandardPaths::TempLocation);
         QVERIFY(QDir::temp().mkpath(aDir));
         const QString aFile = aDir + QStringLiteral("/accessmanagertest-data");
@@ -78,9 +75,6 @@ private Q_SLOTS:
 
     void testPutSequential()
     {
-#if defined(USE_QNAM) && QT_VERSION <= QT_VERSION_CHECK(5, 5, 0)
-        QSKIP("This test is broken with Qt < 5.5, the fix is 9286a8e5dd97c5d4d7e0ed07a73d4ce7240fdc1d in qtbase");
-#endif
         const QString aDir = QStandardPaths::writableLocation(QStandardPaths::TempLocation);
         QVERIFY(QDir::temp().mkpath(aDir));
         const QString aFile = aDir + QStringLiteral("/accessmanagertest-data2");
@@ -113,7 +107,7 @@ private:
      */
     QNetworkAccessManager *manager()
     {
-        static QNetworkAccessManager *ret = Q_NULLPTR;
+        static QNetworkAccessManager *ret = nullptr;
         if (!ret) {
 #ifdef USE_QNAM
             ret = new QNetworkAccessManager(this);
