@@ -19,6 +19,7 @@
 */
 
 #include "knewfilemenu.h"
+#include "../pathhelpers_p.h"
 #include "knameandurlinputdialog.h"
 
 #include <QDialog>
@@ -37,6 +38,7 @@
 #include <QDebug>
 #include <kdesktopfile.h>
 #include <kdirwatch.h>
+#include <kdirnotify.h>
 #include <kjobwidgets.h>
 #include <klocalizedstring.h>
 #include <kmessagebox.h>
@@ -587,7 +589,7 @@ void KNewFileMenuPrivate::executeStrategy()
     QList<QUrl>::const_iterator it = m_popupFiles.constBegin();
     for (; it != m_popupFiles.constEnd(); ++it) {
         QUrl dest = *it;
-        dest.setPath(dest.path() + '/' + KIO::encodeFileName(chosenFileName));
+        dest.setPath(concatPaths(dest.path(), KIO::encodeFileName(chosenFileName)));
 
         QList<QUrl> lstSrc;
         lstSrc.append(uSrc);
@@ -638,6 +640,7 @@ void KNewFileMenuPrivate::fillMenu()
     m_newDirAction = nullptr;
 
     QSet<QString> seenTexts;
+    QString lastTemplatePath;
     // these shall be put at special positions
     QAction *linkURL = nullptr;
     QAction *linkApp = nullptr;
@@ -690,7 +693,11 @@ void KNewFileMenuPrivate::fillMenu()
                     sep->setSeparator(true);
                     menu->addAction(sep);
                 } else {
-
+                    if (lastTemplatePath.startsWith(QDir::homePath()) && !templatePath.startsWith(QDir::homePath())) {
+                        QAction *sep = new QAction(q);
+                        sep->setSeparator(true);
+                        menu->addAction(sep);
+                    }
                     if (!m_supportedMimeTypes.isEmpty()) {
                         bool keep = false;
 
@@ -750,6 +757,7 @@ void KNewFileMenuPrivate::fillMenu()
                     }
                 }
             }
+            lastTemplatePath = entry.templatePath;
         } else { // Separate system from personal templates
             Q_ASSERT(entry.entryType != 0);
 
@@ -773,7 +781,9 @@ void KNewFileMenuPrivate::fillMenu()
             menu->addAction(linkApp);
         }
         Q_ASSERT(m_menuDev);
-        menu->addAction(m_menuDev);
+        if (!m_menuDev->menu()->isEmpty()) {
+            menu->addAction(m_menuDev);
+        }
     }
 }
 
@@ -839,7 +849,6 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
 {
     QUrl url;
     QUrl baseUrl = m_popupFiles.first();
-    bool askAgain = false;
 
     QString name = expandTilde(m_text);
 
@@ -847,6 +856,33 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
         if (QDir::isAbsolutePath(name)) {
             url = QUrl::fromLocalFile(name);
         } else {
+            if (name == QLatin1String(".") || name == QLatin1String("..")) {
+                KGuiItem cancelGuiItem(KStandardGuiItem::cancel());
+                cancelGuiItem.setText(i18nc("@action:button", "Enter a different name"));
+
+                QDialog *confirmDialog = new QDialog(m_parentWidget);
+                confirmDialog->setWindowTitle(i18n("Invalid Directory Name"));
+                confirmDialog->setModal(m_modal);
+                confirmDialog->setAttribute(Qt::WA_DeleteOnClose);
+
+                QDialogButtonBox *buttonBox = new QDialogButtonBox(confirmDialog);
+                buttonBox->setStandardButtons(QDialogButtonBox::Cancel);
+                KGuiItem::assign(buttonBox->button(QDialogButtonBox::Cancel), cancelGuiItem);
+                
+                KMessageBox::createKMessageBox(confirmDialog, buttonBox, QMessageBox::Critical,
+                                   xi18n("Folder <filename>%1</filename> could not be created:<nl/><filename>%1</filename> is reserved for use by the operating system.", name),
+                                   QStringList(),
+                                   QString(),
+                                   nullptr,
+                                   KMessageBox::NoExec,
+                                   QString());
+                
+                QObject::connect(buttonBox, SIGNAL(rejected()), q, SLOT(createDirectory()));
+                m_fileDialog = confirmDialog;
+                confirmDialog->show();
+                _k_slotAbortDialog();
+                return;
+            }
             if (!m_viewShowsHiddenFiles && name.startsWith('.')) {
                 if (!writeHiddenDir) {
                     confirmCreatingHiddenDir(name);
@@ -854,24 +890,22 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
                 }
             }
             url = baseUrl;
-            url.setPath(url.path() + '/' + name);
+            url.setPath(concatPaths(url.path(), name));
         }
     }
 
-    if (!askAgain) {
-        KIO::Job *job = KIO::mkpath(url, baseUrl);
-        job->setProperty("mkpathUrl", url);
-        KJobWidgets::setWindow(job, m_parentWidget);
-        job->ui()->setAutoErrorHandlingEnabled(true);
-        KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkpath, QList<QUrl>(), url, job);
+    // Note that we use mkpath so that a/b/c works.
+    // On the other hand it means that passing the name of a directory that already exists will do nothing.
+    KIO::Job *job = KIO::mkpath(url, baseUrl);
+    job->setProperty("mkpathUrl", url);
+    KJobWidgets::setWindow(job, m_parentWidget);
+    job->uiDelegate()->setAutoErrorHandlingEnabled(true);
+    KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkpath, QList<QUrl>(), url, job);
 
-        if (job) {
-            // We want the error handling to be done by slotResult so that subclasses can reimplement it
-            job->ui()->setAutoErrorHandlingEnabled(false);
-            QObject::connect(job, SIGNAL(result(KJob*)), q, SLOT(slotResult(KJob*)));
-        }
-    } else {
-        q->createDirectory(); // ask again for the name
+    if (job) {
+        // We want the error handling to be done by slotResult so that subclasses can reimplement it
+        job->uiDelegate()->setAutoErrorHandlingEnabled(false);
+        QObject::connect(job, SIGNAL(result(KJob*)), q, SLOT(slotResult(KJob*)));
     }
     _k_slotAbortDialog();
 }
@@ -891,15 +925,36 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     KNewFileMenuSingleton *s = kNewMenuGlobals();
     //qDebug();
 
-    QStringList dirs = { QStringLiteral(":/kio5/newfile-templates") };
-    dirs += QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("templates"), QStandardPaths::LocateDirectory);
+    const QStringList qrcTemplates = { QStringLiteral(":/kio5/newfile-templates") };
+    QStringList installedTemplates = { QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("templates"), QStandardPaths::LocateDirectory) };
+    // Qt does not provide an easy way to receive the xdg dir for templates so we have to find it on our own
+    #ifdef Q_OS_UNIX
+        QString xdgUserDirs = QStandardPaths::locate(QStandardPaths::ConfigLocation, QStringLiteral("user-dirs.dirs"), QStandardPaths::LocateFile);
+        QFile xdgUserDirsFile(xdgUserDirs);
+        if (xdgUserDirsFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            QTextStream in(&xdgUserDirsFile);
+            while (!in.atEnd()) {
+                QString line = in.readLine();
+                if (line.startsWith("XDG_TEMPLATES_DIR=")) {
+                    QString xdgTemplates = line.mid(19, line.size()-20);
+                    xdgTemplates.replace(QString("$HOME"), QDir::homePath());
+                    QDir xdgTemplatesDir(xdgTemplates);
+                    if (xdgTemplatesDir.exists()) {
+                        installedTemplates << xdgTemplates;
+                    }
+                    break;
+                }
+            }
+        }
+    #endif
+
+    const QStringList templates = qrcTemplates + installedTemplates;
 
     // Ensure any changes in the templates dir will call this
     if (! s->dirWatch) {
         s->dirWatch = new KDirWatch;
-        for (QStringList::const_iterator it = dirs.constBegin(); it != dirs.constEnd(); ++it) {
-            //qDebug() << "Templates resource dir:" << *it;
-            s->dirWatch->addDir(*it);
+        for (const QString &dir : installedTemplates) {
+            s->dirWatch->addDir(dir);
         }
         QObject::connect(s->dirWatch, SIGNAL(dirty(QString)),
                          q, SLOT(_k_slotFillTemplates()));
@@ -918,11 +973,11 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     QStringList files;
     QDir dir;
 
-    Q_FOREACH (const QString &path, dirs) {
+    for (const QString &path : templates) {
         dir.setPath(path);
         const QStringList &entryList(dir.entryList(QStringList() << QStringLiteral("*.desktop"), QDir::Files));
         Q_FOREACH (const QString &entry, entryList) {
-            const QString file = dir.path() + QLatin1Char('/') + entry;
+            const QString file = concatPaths(dir.path(), entry);
             files.append(file);
         }
     }
@@ -945,10 +1000,12 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
             QString key = config.desktopGroup().readEntry("Name");
             if (file.endsWith(QLatin1String("Directory.desktop"))) {
                 key.prepend('0');
-            } else if (file.endsWith(QLatin1String("TextFile.desktop"))) {
+            } else if (file.startsWith(QDir::homePath())) {
                 key.prepend('1');
-            } else {
+            } else if (file.endsWith(QLatin1String("TextFile.desktop"))) {
                 key.prepend('2');
+            } else {
+                key.prepend('3');
             }
             EntryWithName en = { key, e };
             if (ulist.contains(url)) {
@@ -1228,7 +1285,7 @@ void KNewFileMenu::setViewShowsHiddenFiles(bool b)
 void KNewFileMenu::slotResult(KJob *job)
 {
     if (job->error()) {
-        static_cast<KIO::Job *>(job)->ui()->showErrorMessage();
+        static_cast<KIO::Job *>(job)->uiDelegate()->showErrorMessage();
     } else {
         // Was this a copy or a mkdir?
         KIO::CopyJob *copyJob = ::qobject_cast<KIO::CopyJob *>(job);
@@ -1242,6 +1299,7 @@ void KNewFileMenu::slotResult(KJob *job)
             emit fileCreated(destUrl);
         } else if (KIO::SimpleJob *simpleJob = ::qobject_cast<KIO::SimpleJob *>(job)) {
             // Called in the storedPut() case
+            org::kde::KDirNotify::emitFilesAdded(simpleJob->url().adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
             emit fileCreated(simpleJob->url());
         }  else {
             // Can be mkdir
